@@ -7,7 +7,7 @@ import os
 import time
 
 from math import floor
-from typing import Dict, Union
+from typing import Any, Callable, Dict, Union
 
 import yt_dlp as youtube_dl
 
@@ -18,6 +18,7 @@ from pyrogram.types import CallbackQuery, Message
 # from youtube_dl.utils import DownloadError, GeoRestrictedError
 from yt_dlp.utils import DownloadError, GeoRestrictedError
 
+from iytdl.exceptions import DownloadFailedError
 from iytdl.processes import Process
 from iytdl.utils import *
 
@@ -26,20 +27,10 @@ logger = logging.getLogger(__name__)
 
 
 class Downloader:
-    async def video_downloader(self, url: str, uid: str, rnd_key: str, prog_func):
-        """YouTubeDL Video Downloader
+    async def video_downloader(
+        self, url: str, uid: str, rnd_key: str, prog_func: Callable
+    ) -> Union[int, str]:
 
-        Parameters:
-        ----------
-            - url (`str`): youtube-dl supported URL.
-            - uid (`str`): format.
-            - rnd_key (`str`): Unique Key.
-            - prog_func (`[type]`): progress Hook.
-
-        Returns:
-        -------
-            `[type]`: [description]
-        """
         options = {
             "addmetadata": True,
             "geo_bypass": True,
@@ -56,13 +47,11 @@ class Downloader:
             "quiet": self.silent,
             "logtostderr": self.silent,
         }
-
-        if (ext_dl := self.external_downloader) is not None:
-            options.update(ext_dl._export())
         return await self.ytdownloader(url, options)
 
-    async def audio_downloader(self, url: str, uid: str, rnd_key: str, prog_func):
-        logger.info(f"[Seleced Audio Quality => {uid}]")
+    async def audio_downloader(
+        self, url: str, uid: str, rnd_key: str, prog_func: Callable
+    ) -> Union[int, str]:
         options = {
             "outtmpl": os.path.join(
                 str(self.download_path), rnd_key, "%(title)s-%(format)s.%(ext)s"
@@ -86,12 +75,14 @@ class Downloader:
             "quiet": self.silent,
             "logtostderr": self.silent,
         }
-        if (ext_dl := self.external_downloader) is not None:
-            options.update(ext_dl._export())
         return await self.ytdownloader(url, options)
 
     @run_sync
-    def ytdownloader(self, url: str, options: Dict):
+    def ytdownloader(self, url: str, options: Dict[str, Any]) -> Union[int, str]:
+        if self._ffmpeg != "ffmpeg":
+            options["ffmpeg_location"] = str(self._ffmpeg)
+        if (ext_dl := self.external_downloader) is not None:
+            options.update(ext_dl._export())
         try:
             with youtube_dl.YoutubeDL(options) as ytdl:
                 return ytdl.download([url])
@@ -133,7 +124,7 @@ class Downloader:
         ------
             `TypeError`: On unsupported `downtype`
             `StopTransmission`: When download is cancelled
-            `DownloadError`: In case youtube_dl download return code is not equal to 0
+            `DownloadFailedError`: In case youtube_dl download return code is not equal to 0
         """
         last_update_time = None
 
@@ -196,7 +187,7 @@ class Downloader:
 
         if isinstance(out, int) and out == 0:
             return key
-        raise DownloadError(str(out))
+        raise DownloadFailedError(str(out))
 
     @staticmethod
     async def progress_func(process: Process, text: str) -> None:
@@ -213,5 +204,5 @@ class Downloader:
             raise p_e
         except (ContinuePropagation, MessageNotModified):
             pass
-        except Exception as e:
-            logger.error(format_exception(e))
+        except Exception:
+            logger.exception("Progress")
