@@ -27,6 +27,18 @@ class Extractor:
 
     @run_sync
     def generic_extractor(self, key: str, url: str) -> Optional[SearhResult]:
+        """Generic extractor for URLs other than YouTube
+        [more info](https://github.com/yt-dlp/yt-dlp/blob/master/supportedsites.md).
+
+        Parameters:
+        ----------
+            - key (`str`): Unique key for Callback.
+            - url (`str`): Http URL.
+
+        Returns:
+        -------
+            `Optional[SearhResult]`: On Success
+        """
         # passing key as we can't pass the entire url in callback_data
         buttons = [
             [
@@ -110,6 +122,16 @@ class Extractor:
         )
 
     def filter_generic_formats(self, raw_formats: Dict) -> Dict:
+        """Filter Formats Based on 'tbr', 'width' and 'acodec'
+
+        Parameters:
+        ----------
+            - raw_formats (`Dict`): Raw youtube-dl formats.
+
+        Returns:
+        -------
+            `Dict`: Filtered Formats
+        """
         widthset = set()
 
         def qual_filter(frmt) -> bool:
@@ -130,14 +152,24 @@ class Extractor:
         return frmt_list if len(frmt_list) > 1 else raw_formats
 
     @run_sync
-    def get_download_button(self, yt_id: str):
+    def get_download_button(self, yt_id: str) -> SearhResult:
+        """Generate Inline Buttons for YouTube Video
+
+        Parameters:
+        ----------
+            - yt_id (`str`): YouTube video key.
+
+        Returns:
+        -------
+            `SearhResult`: `~iytdl.types.SearhResult`
+        """
         buttons = [
             [
                 InlineKeyboardButton(
                     "⭐️ BEST - 📹 MKV", callback_data=f"yt_dl|{yt_id}|mkv|v"
                 ),
                 InlineKeyboardButton(
-                    "⭐️ BEST - 📹 WebM/MP4",
+                    "⭐️ BEST - 📹 MP4",
                     callback_data=f"yt_dl|{yt_id}|mp4|v",
                 ),
             ]
@@ -217,32 +249,59 @@ class Extractor:
 
     @staticmethod
     def get_choice_by_id(
-        choice_id: str, media_type: str, yt_url: bool = True
+        choice_id: str, media_type: str, yt_url: bool = True, max_filesize: int = 1950
     ) -> Tuple[str]:
+        """Youtube-dl downloader formats for video / audio
+
+        Parameters:
+        ----------
+            - choice_id (`str`): Format choice.
+            - media_type (`str`): `"video"` or `"audio"`.
+            - yt_url (`bool`, optional): If URL is from https://www.youtube.com/. (Defaults to `True`)
+            - max_filesize (`int`, optional): Max video filesize (in MB).
+
+        Returns:
+        -------
+            `Tuple[str]`: (Quality format, Display str)
+        """
+        filesize_flt = f"[filesize<?{max_filesize}M]"  # Max filesize
         if choice_id == "mkv":
-            # Download and Merge (best video) + (best audio)
-            choice_str = "(bestvideo+bestaudio/best)[filesize<?1950M]"
-            disp_str = "Best (video+audio)"
+            # Overall Best format
+            # - can have any video format except `.webm` as it is uploaded as document.
+            choice_str = f"(bestvideo+bestaudio/best)[ext!=?webm]{filesize_flt}"
+            disp_str = "[ 🎵 + 📹 ]  Best"
         elif choice_id == "mp4":
-            # Download best Mp4 format if available
-            # Webm is not supported by Telegram anymore :(
+            # Best streamable format i.e `.mp4`
+            disp_str = "[ 🎵 + 📹 ]  Best MP4"
             if yt_url:
-                choice_str = "(bestvideo[ext=mp4]+(258/256/bestaudio[ext=m4a])/best[ext=mp4]/best)[filesize<?1950M]"
-                disp_str = "Best MP4"
+                choice_str = (
+                    "(bestvideo[ext=mp4]+(258/256/bestaudio[ext=m4a])"
+                    f"/best[ext=mp4]/best[ext!=webm]){filesize_flt}"
+                )
             else:
-                choice_str = "(bestvideo[ext=mp4]+bestaudio[ext=m4a]/best[ext=mp4]/best)[filesize<?1950M]"
-                disp_str = "Best MP4 (video+audio)"
+                choice_str = (
+                    "(bestvideo[ext=?mp4]+bestaudio[ext=?m4a]"
+                    f"/best[ext=?mp4]/best[ext!=?webm]/best){filesize_flt}"
+                )
         elif choice_id == "mp3":
+            # Best audio quality upscaled to 320 kbps
             choice_str = "320"
-            disp_str = "320 Kbps"
+            disp_str = "[ 🎵 ]  320 Kbps"
         else:
-            disp_str = choice_id
+            disp_str = f"[ 🎵 ]  {choice_id}"
             if media_type == "v":
-                # mp4 video quality + best compatible audio
+                # Merge best compatible audio with choosen video quality
                 if yt_url:
-                    choice_str = f"({disp_str}+(258/256/bestaudio[ext=m4a])/best[ext=mp4]/best)[filesize<?1950M]"
+                    choice_str = (
+                        f"({choice_id}+(258/256/bestaudio[ext=?m4a]/bestaudio)"
+                        f"/{choice_id}/best[ext=mp4]/best)[ext!=?webm]{filesize_flt}"
+                    )
                 else:
-                    choice_str = f"({disp_str}+bestaudio[ext=m4a]/best[ext=mp4]/best)[filesize<?1950M]"
-            else:  # Audio
-                choice_str = disp_str
+                    choice_str = (
+                        f"({choice_id}+bestaudio/{choice_id}/"
+                        f"best[ext=?mp4]/best)[ext!=?webm]{filesize_flt}"
+                    )
+            else:
+                # Choosen audio quality
+                choice_str = choice_id
         return choice_str, disp_str
