@@ -4,7 +4,7 @@ import asyncio
 import hashlib
 import re
 
-from pathlib import Path
+from pathlib import Path, WindowsPath
 from typing import Optional, Tuple, Union
 
 from aiohttp import ClientSession
@@ -20,7 +20,7 @@ from iytdl.extractors import Extractor
 from iytdl.formatter import ResultFormatter, gen_search_markup
 from iytdl.sql_cache import AioSQLiteDB
 from iytdl.upload_lib.uploader import Uploader
-from iytdl.utils import run_sync
+from iytdl.utils import run_command, run_sync
 
 
 class iYTDL(Extractor, Downloader, Uploader):
@@ -35,8 +35,9 @@ class iYTDL(Extractor, Downloader, Uploader):
         cache_path: str = "",
         delete_media: bool = False,
         external_downloader: Optional[types.ExternalDownloader] = None,
+        ffmpeg_location: str = "ffmpeg",
     ) -> None:
-        """Main Class
+        """Main class
 
         Parameters:
         ----------
@@ -49,14 +50,16 @@ class iYTDL(Extractor, Downloader, Uploader):
             - cache_path (`str`, optional): Path to store cache. (Defaults to `""`)
             - delete_media: (`bool`, optional): Delete media from local storage after uploading on Telegram. (Defaults to `False`)
             - external_downloader: (`Optional[types.ExternalDownloader]`, optional): External Downloader e.g `types.external_downloader.Aria2c`. (Defaults to `None`)
+            - ffmpeg_location (`str`, optional): Custom location for FFMPEG. (Defaults to `"ffmpeg"`)
         """
         self.yt_link_regex = re.compile(
-            r"(?:youtube\.com|youtu\.be)/(?:[\w-]+\?v=|embed/|v/|shorts/)?([\w-]{11})"
+            r"(?:youtube(?:-nocookie)?\.com|youtu\.be)/(?:[\w-]+\?v=|embed/|v/|shorts/)?([\w-]{11})"
         )
         self.generic_url_regex = re.compile(r"^https?://\S+")
         self.default_thumb = default_thumb
         self.http = session or ClientSession()
         _cache_path = Path(cache_path)
+        _cache_path.mkdir(exist_ok=True, parents=True)
         if _cache_path.is_file():
             raise TypeError(f"'{cache_path}' expected a Directory got a File instead")
         self.cache = AioSQLiteDB(
@@ -69,11 +72,35 @@ class iYTDL(Extractor, Downloader, Uploader):
         self.download_path.mkdir(exist_ok=True, parents=True)
         self.external_downloader = external_downloader
         self.delete_file_after_upload = delete_media
-
+        if ffmpeg_location != "ffmpeg":
+            ffmpeg_location = Path(ffmpeg_location)
+            if not ffmpeg_location.is_file():
+                raise FileNotFoundError(ffmpeg_location)
+        self._ffmpeg = ffmpeg_location
         super().__init__(silent=silent)
 
     @classmethod
     async def init(cls, *args, **kwargs) -> "iYTDL":
+        """Utility method to create new class instance, Note: no need to start()
+
+        Parameters:
+        ----------
+            - log_group_id (`Union[int, str]`):  Log channel / group id to upload downloaded media.
+            - session (`Optional[ClientSession]`, optional): Aiohttp ClientSession. (Defaults to `None`)
+            - silent (`bool`, optional): Disable youtube_dl stdout. (Defaults to `False`)
+            - download_path (`str`, optional): Custom download location. (Defaults to `"downloads"`)
+            - loop (`Optional[asyncio.AbstractEventLoop]`, optional): Event loop. (Defaults to `None`)
+            - default_thumb (`str`, optional): Fallback thumbnail. (Defaults to `"https://i.imgur.com/4LwPLai.png"`)
+            - cache_path (`str`, optional): Path to store cache. (Defaults to `""`)
+            - delete_media: (`bool`, optional): Delete media from local storage after uploading on Telegram. (Defaults to `False`)
+            - external_downloader: (`Optional[types.ExternalDownloader]`, optional): External Downloader e.g `types.external_downloader.Aria2c`. (Defaults to `None`)
+            - ffmpeg_location (`str`, optional): Custom location for FFMPEG. (Defaults to `"ffmpeg"`)
+
+        Returns:
+        -------
+            `~iytdl.iYTDL`
+        """
+
         yt = cls(*args, **kwargs)
         await yt.start()
         return yt
@@ -305,6 +332,24 @@ class iYTDL(Extractor, Downloader, Uploader):
             link = self.default_thumb
         return link
 
+    async def _check_ffmpeg(self) -> None:
+        if isinstance(self._ffmpeg, Path):
+            ffmpeg = self._ffmpeg
+            _ffprobe = self._ffmpeg.parent.joinpath(
+                f"ffprobe{'.exe' if isinstance(self._ffmpeg, WindowsPath) else ''}"
+            )
+            ffprobe = _ffprobe if _ffprobe.is_file() else None
+        else:
+            ffmpeg = "ffmpeg"
+            ffprobe = "ffprobe"
+        out_1 = await run_command(f"{ffmpeg} -version", shell=True)
+        if out_1[1] != 0:
+            raise ValueError(f"'{ffmpeg}' was not Found !")
+        if ffprobe is not None:
+            out_2 = await run_command(f"{ffprobe} -version", shell=True, silent=True)
+            if out_2[1] == 0:
+                setattr(self, "_ffprobe", ffprobe)
+
     async def stop(self) -> None:
         """Stop iYTDL instance manually or Use Context Manager"""
         if self.http and not self.http.closed:
@@ -313,6 +358,7 @@ class iYTDL(Extractor, Downloader, Uploader):
 
     async def start(self) -> None:
         """Start iYTDL instance manually or Use Context Manager"""
+        await self._check_ffmpeg()
         await self.cache._init()
 
     async def __aenter__(self) -> "iYTDL":
